"use strict";
/* Type checks */

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isNamedFunction = exports.hasName = exports.isValidTarget = exports.isValidSwap = exports.isValidSource = exports.isValidChild = exports.isEBObject = exports.isElement = exports.isEventTupleArray = exports.isEventTuple = exports.isStringObject = exports.isArrayArray = exports.isArray = exports.isFunction = exports.isString = exports.isNumber = exports.isTypeOf = exports.typeOf = void 0;

exports.typeOf = function (input) {
  return "".concat({}.toString.call(input)).replace(/^\[object ([a-z]+)\]$/i, '$1').toLowerCase();
};

exports.isTypeOf = function (input) {
  for (var _len = arguments.length, types = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    types[_key - 1] = arguments[_key];
  }

  return types.map(function (type) {
    return exports.typeOf(input) === type.toLowerCase();
  }).reduce(function (a, c) {
    return a || c;
  });
};

exports.isNumber = function (input) {
  return exports.isTypeOf(input, 'number');
};

exports.isString = function (input) {
  return exports.isTypeOf(input, 'string');
};

exports.isFunction = function (input) {
  return exports.isTypeOf(input, 'function');
};

exports.isArray = function (input) {
  return exports.isTypeOf(input, 'array');
};

exports.isArrayArray = function (input) {
  return exports.isTypeOf(input, 'array') && input.every(function (item) {
    return exports.isTypeOf(item, 'array');
  });
};

exports.isStringObject = function (input) {
  return exports.isTypeOf(input, 'object') && Object.keys(input).every(function (item) {
    return exports.isTypeOf(item, 'string');
  });
};

exports.isEventTuple = function (input) {
  return exports.isTypeOf(input, 'array') && exports.isTypeOf(input[0], 'string') && exports.isTypeOf(input[1], 'function') && (!input[2] || exports.isTypeOf(input[2], 'boolean', 'object'));
};

exports.isEventTupleArray = function (input) {
  return exports.isTypeOf(input, 'array') && input.every(exports.isEventTuple);
};

exports.isElement = function (input) {
  return input instanceof Element;
};

exports.isEBObject = function (input) {
  return input instanceof Object && input.isEBuilder;
};

exports.isValidChild = function (input) {
  return exports.isTypeOf(input, 'string', 'number') || input instanceof Node || input.isEBuilder;
};

exports.isValidSource = function (source) {
  return exports.isString(source) || source instanceof Element || source instanceof ShadowRoot;
};

exports.isValidSwap = function (element, swapped) {
  return element instanceof HTMLElement && swapped instanceof HTMLElement && !!element.parentNode && !!swapped.parentNode && element !== swapped;
};

exports.isValidTarget = function (target) {
  return exports.isEBObject(target) || target instanceof Element || target instanceof ShadowRoot;
};

exports.hasName = function (input) {
  return !!input.name;
};

exports.isNamedFunction = function (input) {
  return exports.isFunction(input) && exports.hasName(input);
};